<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

if (!defined('ABSPATH')) {
    exit;
}

if (!function_exists('acoofm_empty')) {

    /**
     * Check a variable is empty
     * @since 1.0.0
     * @param string|integer|array|float 
     * @return boolean
     */
    function acoofm_empty($var)
    {
        if (is_array($var)) {
            return empty($var);
        } else {
            return ($var === null || $var === false || $var === '');
        }
    }

}

if (!function_exists('acoofm_update_option')) {

    /**
     * Function To update Plugin Specific Wordpress Option
     * @since 1.0.0
     * @return boolean
     */
    function acoofm_update_option($key, $options){
        $current_options = get_option( ACOOFM_OPTION_NAME, array() );
        if(isset($current_options[$key])) {
            if($current_options[$key] === $options) {
                return true;
            }
        }
        $current_options[$key] = $options;
        return update_option( ACOOFM_OPTION_NAME, $current_options, true);
    }

}

if (!function_exists('acoofm_get_option')) {

    /**
     * Function To get Plugin Specific Wordpress Option
     * @since 1.0.0
     * @return array|boolean|string|integer|float|double
     */
    function acoofm_get_option($key, $default=false){
        $current_options = get_option( ACOOFM_OPTION_NAME, array() );
        if(isset($current_options[$key])){
            return $current_options[$key];
        } else {
            return $default;
        }
    }

}

if (!function_exists('acoofm_get_service')) {

    /**
     * Function To get Current Service
     * @since 1.0.0
     * @return array|boolean|string|integer|float|double
     */
    function acoofm_get_service($option='', $default=false){
        $current_service = acoofm_get_option('service');
        if(isset($current_service) && !empty($current_service)){
            if(isset($option) && !empty($option)){
                if(isset($current_service[$option])) {
                    return $current_service[$option];
                } else {
                    return $default;
                }
            } else {
                return $current_service;
            }
        } else {
            return $default;
        }
    }

}

if (!function_exists('acoofm_get_settings')) {

    /**
     * Function To get Current settings
     * @since 1.0.0
     * @return array|boolean|string|integer|float|double
     */
    function acoofm_get_settings($option='', $default=false){
        $current_setttings = acoofm_get_option('settings');
        if(isset($current_setttings) && !empty($current_setttings)){
            if(isset($option) && !empty($option)){
                if(isset($current_setttings[$option])) {
                    return $current_setttings[$option];
                } else {
                    return $default;
                }
            } else {
                return $current_setttings;
            }
        } else {
            return $default;
        }
    }
}

if (!function_exists('acoofm_is_service_enabled')) {

    /**
     * Function To check service is enabled
     * @since 1.0.0
     * @return array|boolean|string|integer|float|double
     */
    function acoofm_is_service_enabled(){
        $service    = acoofm_get_option('service', false);

        if(
            isset($service) && !empty($service) &&
            isset($service['slug']) && !empty($service['slug']) 
        ) {
            return true;
        }
        return false;
    }

}


if (!function_exists('acoofm_is_copy_to_server')) {

    /**
     * Function To check is setting is enabled to copy to server
     * @since 1.0.0
     * @return array|boolean|string|integer|float|double
     */
    function acoofm_is_copy_to_server(){
        $settings   = acoofm_get_option('settings', false);
        if(
            isset($settings) && !empty($settings) &&
            isset($settings['copy_to_bucket']) && $settings['copy_to_bucket'] 
        ) {
            return true;
        }
        return false;
    }
}

if (!function_exists('acoofm_is_rewrite_url')) {

    /**
     * Function To check is setting is enabled for rewriting URL
     * @since 1.0.0
     * @return array|boolean|string|integer|float|double
     */
    function acoofm_is_rewrite_url(){
        $settings   = acoofm_get_option('settings', false);
        if(
            isset($settings) && !empty($settings) &&
            isset($settings['rewrite_url']) && $settings['rewrite_url'] 
        ) {
            return true;
        }
        return false;
    }
}

if (!function_exists('acoofm_maybe_convert_size_to_string')) {

    /**
     * Maybe convert size to string
     *
     * @param int   $attachment_id
     * @param mixed $size
     *
     * @return null|string
     */
    function acoofm_maybe_convert_size_to_string( $attachment_id, $size ) {
        if ( is_array( $size ) ) {
            $width  = ( isset( $size[0] ) && $size[0] > 0 ) ? $size[0] : 1;
			$height = ( isset( $size[1] ) && $size[1] > 0 ) ? $size[1] : 1;
			$original_aspect_ratio = $width / $height;
			$meta   = wp_get_attachment_metadata( $attachment_id );

			if ( ! isset( $meta['sizes'] ) || empty( $meta['sizes'] ) ) {
				return false;
			}

			$sizes = $meta['sizes'];
			uasort( $sizes, function ( $a, $b ) {
				// Order by image area
				return ( $a['width'] * $a['height'] ) - ( $b['width'] * $b['height'] );
			} );

			$near_matches = array();

			foreach ( $sizes as $size => $value ) {
				if ( $width > $value['width'] || $height > $value['height'] ) {
					continue;
				}
				$aspect_ratio = $value['width'] / $value['height'];
				if ( $aspect_ratio === $original_aspect_ratio ) {
					return $size;
				}
				$near_matches[] = $size;
			}
			// Return nearest match
			if ( ! empty( $near_matches ) ) {
				return $near_matches[0];
			}
        }

        return $size;
    }
}


if (!function_exists('acoofm_get_attachment_relative_path')) {

    /**
     * Get Image URL and path from URL
     * Return Relative URL and Path of an attachment.
     * @since 1.0.0
     * 
     */
    function acoofm_get_attachment_relative_path($file) {
        if ( isset($file) && !empty($file) ) {
            $uploads            = wp_get_upload_dir();
            $file_path          = '';
            $site_url           = site_url('/');
            $server_base_path   = acoofm_get_settings('base_path');
            if ( $uploads && false === $uploads['error'] ) {

                $uploadDir  = substr( $uploads['baseurl'], strpos( $uploads['baseurl'], $site_url ) + strlen($site_url)); 

                // Get URL and PATH
                if ( 0 === strpos( $file, $uploads['basedir'] ) ||  0 === strpos( $file, $uploads['baseurl'] ) ) {   // If URL is full link 
                    $file_path  = str_replace( $uploads['basedir'], '', $file );
                    $file_path  = str_replace( $uploads['baseurl'], '', $file_path ); // Replace if has URl
                } else if ( 
                    0 === strpos( $file, str_replace('/','\\', $uploads['basedir'] )) ||  
                    0 === strpos( $file, str_replace('\\','/', $uploads['baseurl'] )) 
                ) {   // If URL is full link and the url is slash unified (Like: str_replace('/','\\', $dir))
                    $file_path  = str_replace( str_replace('/','\\', $uploads['basedir'] ), '', $file );
                    $file_path  = str_replace( str_replace('\\','/', $uploads['baseurl'] ), '', $file_path ); // Replace if has URl
                    $file_path  = str_replace('\\','/', $file_path );
                } else if ( false !== strpos( $file, $uploadDir ) ) {   //If URL has sub Directory That matches end of base URL(eg: wp-content/uploads)
                    $fileDir    = dirname( $file );
                    $start_pos  = strpos( $fileDir, $uploadDir ) + strlen($uploadDir); 
                    $subDir     = substr( $fileDir, $start_pos, strlen($fileDir)); // Find Sub Directory
                    $file_name  = wp_basename( $file );

                    $file_path   = trailingslashit($subDir) . $file_name;
                } else if(false !== strpos( $file, trailingslashit($server_base_path))) {
                    $fileDir    = dirname( $file );
                    $start_pos  = strpos( $fileDir, $server_base_path) + strlen($server_base_path); 
                    $subDir     = substr( $fileDir, $start_pos, strlen($fileDir)); // Find Sub Directory
                    $file_name  = wp_basename( $file );

                    $file_path   = trailingslashit($subDir) . $file_name;
                } else {
                    $file_path  = $file;
                }

                return apply_filters( 'acoofm_get_relative_file_path_from_upload_directory', untrailingslashit(ltrim( $file_path, '/\\' )), $file );
            } 
        } 
        return false;
    }
}



if (!function_exists('acoofm_generate_object_key')) {

    /**
     * Generate Key for Objects
     * @since 1.0.0
     */
    function acoofm_generate_object_key($file_name, $prefix) {
        $upload_path = '';
        $base_path = acoofm_get_settings('base_path');
        $year_month = acoofm_get_settings('year_month');

        if(isset($base_path) && !empty($base_path)) {
            $upload_path.= preg_replace('~/+~', '/', 
                                    str_replace('\\', '/', 
                                        trim($base_path," \n\r\t\v\x00\/ ")
                                    )
                                );
        }
        if(isset($year_month) && $year_month) {
            $upload_path.= '/'.date("Y/m");
        }
        if($upload_path == '') {
            return $upload_path.= $prefix.$file_name;
        }
        return $upload_path.= '/'.$prefix.$file_name;
    }
}


if (!function_exists('acoofm_is_extension_available')) {

    /**
     * Check extension is compatible
     * @since 1.0.0
     * @return boolean
     */
    function acoofm_is_extension_available($extension){
        if(defined('ALLOW_UNFILTERED_UPLOADS') && ALLOW_UNFILTERED_UPLOADS) {
            return true;
        }

        $mime_types = wp_get_mime_types();

        $filtered = array_filter(
            $mime_types,
            function ($ext) use ($extension) {
                return in_array($extension, explode("|",$ext));
            },
            ARRAY_FILTER_USE_KEY
        );
       
        if(isset($filtered) && !empty($filtered)) {
            return true;
        }

        return false;
    }

}


if (!function_exists('acoofm_check_existing_file_names')) {

    /**
     * Check whether a file exist in a list of files
     * @since 1.0.1
     * @return boolean
     */
    function acoofm_check_existing_file_names( $filename, $files ) {
        $fname = pathinfo( $filename, PATHINFO_FILENAME );
        $ext   = pathinfo( $filename, PATHINFO_EXTENSION );
    
        // Edge case, file names like `.ext`.
        if ( empty( $fname ) ) {
            return false;
        }
    
        if ( $ext ) {
            $ext = ".$ext";
        }
    
        $regex = '/^' . preg_quote( $fname ) . '-(?:\d+x\d+|scaled|rotated)' . preg_quote( $ext ) . '$/i';
    
        foreach ( $files as $file ) {
            if ( preg_match( $regex, wp_basename($file) ) ) {
                return true;
            }
        }
    
        return false;
    }
}


if (!function_exists('acoofm_create_no_query_url')) {

    /**
     * Create URL Without Query 
     * @since 1.0.0
     * @return boolean
     */
    function acoofm_create_no_query_url($parsed_url){
        $scheme   = isset($parsed_url['scheme']) ? $parsed_url['scheme'] . '://' : '';
        $host     = isset($parsed_url['host']) ? $parsed_url['host'] : '';
        $port     = isset($parsed_url['port']) ? ':' . $parsed_url['port'] : '';
        $user     = isset($parsed_url['user']) ? $parsed_url['user'] : '';
        $pass     = isset($parsed_url['pass']) ? ':' . $parsed_url['pass']  : '';
        $pass     = ($user || $pass) ? "$pass@" : '';
        $path     = isset($parsed_url['path']) ? $parsed_url['path'] : '';
        return "$scheme$user$pass$host$port$path";
    }

}
